// ***************************************************************************
// main
// ***************************************************************************
// This file was developed by Prof. Dr. Peer Johannsen as part of the
// "Retro-Programming" and "Advanced C Programming" class at
// Pforzheim University, Germany.
//
// It can freely be used, but at one's own risk and for non-commercial
// purposes only. Please respect the copyright and credit the origin of
// this file.
//
// Feedback, suggestions and bug-reports are welcome and can be sent to:
// peer.johannsen@pforzheim-university.de
// ---------------------------------------------------------------------------

#include <vectrex.h>
#include <3dForC.h>
#include <ymPlayerOptimSpeed.h>
#include "STARWARS.h"

// ---------------------------------------------------------------------------
// cold reset: the vectrex logo is shown, all ram data is cleared
// warm reset: skip vectrex logo and keep ram data
// ---------------------------------------------------------------------------
// at system startup, when powering up the vectrex, a cold reset is performed
// if the reset button is pressed, then a warm reset is performed
// ---------------------------------------------------------------------------
// after each reset, the cartridge title is shown and then main() is called
// ---------------------------------------------------------------------------

const struct Line3d missile1[]=
{
	{255, _1_1_0},
	{255, _0_1_0},
	{255, _0_1_0},
	{255, _1_1_N},
	{255, _1_1_N},
	{255, _N_0_1},
	{255, _N_0_1},
	{255, _N_N_0},
	{255, _N_1_0},
	{255, _1_N_1},
	{255, _0_N_0},
	{255, _0_N_0},
	{255, _0_N_0},
	{255, _0_N_N},
	{255, _N_1_0},
	{255, _0_1_0},
	{255, _0_1_0},
	{255, _N_1_N},
	{255, _N_1_N},
	{255, _1_0_1},
	{255, _1_0_1},
	{  0, _1_N_1},
	{255, _1_1_N},
	{  1, _0_0_N}
};


#define MAX_OBJECTS 3

struct Object
{
	signed int x;
	signed int y;
	unsigned int angle_x; // dependend on x position
	unsigned int angle_y; // dependend on x position
	unsigned int angle_z; // dependend on x position
	unsigned int distance;
	unsigned int scale;
	unsigned long flags3d;
	void *object;
};
struct Object missiles[MAX_OBJECTS];

void initObjects()
{
	for (int i=0; i< MAX_OBJECTS;i++)
	{
		missiles[i].x = 0;
		missiles[i].angle_x = 0;
		missiles[i].angle_y = 0;
		missiles[i].angle_z = 0;
		missiles[i].distance = 0;
		missiles[i].scale = 0;
		missiles[i].flags3d = 0 + TEST_1_1_0 + TEST_1_0_1 + TEST_0_1_0 + TEST_0_1_1 + TEST_N_1_0 + TEST_N_0_1 + TEST_1_N_1 + TEST_1_1_N;
		missiles[i].object = 0;
	}
}

void initOneStructure(unsigned int i)
{
	missiles[i].x =(signed int) Random();
	missiles[i].y = (signed int) Random();
	missiles[i].angle_x = (unsigned int) Random() & 64;
	missiles[i].angle_y = (unsigned int) Random() & 64;
	missiles[i].angle_z = (unsigned int) Random() & 64;
	missiles[i].distance = (unsigned int) Random() & 0x20;
	missiles[i].scale = 5;
	missiles[i].object = (void *)missile1;
}


void plot3d()
{
	int initDone = 0;
	for (unsigned int i=0; i< MAX_OBJECTS;i++)
	{
		if (missiles[i].object != 0)
		{
			missiles[i].distance++;
			if (missiles[i].distance>=127)
			{
				missiles[i].distance = 0;
				missiles[i].object = (void *)0;
				continue;
			}
			
			if ((missiles[i].distance &0x03) == 0)
			{
				missiles[i].scale++;
			}
			missiles[i].angle_x++;
			if (missiles[i].angle_x>=64) missiles[i].angle_x-=64;
			missiles[i].angle_y++;
			if (missiles[i].angle_y>=64) missiles[i].angle_y-=64;
			missiles[i].angle_z++;
			if (missiles[i].angle_z>=64) missiles[i].angle_z-=64;
			
			if (missiles[i].x>0) missiles[i].x--;
			if (missiles[i].x<0) missiles[i].x++;
			if (missiles[i].y>0) missiles[i].y--;
			if (missiles[i].y<0) missiles[i].y++;
			
			
			angle_x = missiles[i].angle_x;
			angle_y = missiles[i].angle_y;
			angle_z = missiles[i].angle_z;
			
			
			VIA_t1_cnt_lo = (unsigned int)0x60;
			Moveto_d(missiles[i].y, missiles[i].x);
			vectorBits = missiles[i].flags3d;
			
			init3d_all();
			
			VIA_t1_cnt_lo = missiles[i].scale;
			Intensity_a(missiles[i].distance);
			
			draw_3d(missiles[i].object); // casting to pointer type of function necessary (const -> volatile haha!)
			Reset0Ref();
		}
		else if (initDone == 0)
		{
			initDone = 1;
			initOneStructure(i);
		}
	}
}

int main(void)
{
	initObjects();
	while(1)
	{
		if (ym_data_current == 0)
		{
			ym_init(&STARWARS_data);
		}
		ym_sound();
		Wait_Recal();
		Do_Sound();
		plot3d();
	}
	
	// if return value is <= 0, then a warm reset will be performed,
	// otherwise a cold reset will be performed
	return 0;
}

// ***************************************************************************
// end of file
// ***************************************************************************
